@@ -10,6 +10,7 @@ import yaml
 import os
 import json
 from datacenter_utils.logger import logger
+from io import StringIO
 
 
 def read_yaml(file_path, key='pg'):
@@ -76,7 +77,6 @@ class Postgres(object):
         logger.info('{} is_table_exist: {}'.format(table_str, exist))
         return exist
 
-
     def upsert_df(self, df, table_name, constraint_columns,
                   date_columns=None, timestamp_columns=None
                   ):
@@ -97,10 +97,10 @@ class Postgres(object):
         """
         conf = self.conf
         conn = psycopg2.connect(database=conf['database'],
-                                    user=conf['user'],
-                                    password=conf['password'],
-                                    host=conf['host'],
-                                    port=int(conf['port']))
+                                user=conf['user'],
+                                password=conf['password'],
+                                host=conf['host'],
+                                port=int(conf['port']))
         cursor = conn.cursor()
         data_list = df.values.tolist()
         columns = list(df.columns)
@@ -200,10 +200,10 @@ class Postgres(object):
         """
         conf = self.conf
         conn = psycopg2.connect(database=conf['database'],
-                                    user=conf['user'],
-                                    password=conf['password'],
-                                    host=conf['host'],
-                                    port=int(conf['port']))
+                                user=conf['user'],
+                                password=conf['password'],
+                                host=conf['host'],
+                                port=int(conf['port']))
         cursor = conn.cursor()
         data_list = df.values.tolist()
         columns = list(df.columns)
@@ -351,10 +351,10 @@ class Postgres(object):
         """
         conf = self.conf
         conn = psycopg2.connect(database=conf['database'],
-                                    user=conf['user'],
-                                    password=conf['password'],
-                                    host=conf['host'],
-                                    port=int(conf['port']))
+                                user=conf['user'],
+                                password=conf['password'],
+                                host=conf['host'],
+                                port=int(conf['port']))
         cursor = conn.cursor()
         if columns:
             columns_str = ",".join(['"{}"'.format(i) for i in columns])
@@ -392,10 +392,10 @@ class Postgres(object):
         """
         conf = self.conf
         conn = psycopg2.connect(database=conf['database'],
-                                    user=conf['user'],
-                                    password=conf['password'],
-                                    host=conf['host'],
-                                    port=int(conf['port']))
+                                user=conf['user'],
+                                password=conf['password'],
+                                host=conf['host'],
+                                port=int(conf['port']))
         cursor = conn.cursor()
         if columns:
             columns_str = ",".join(['"{}"'.format(i) for i in columns])
@@ -412,7 +412,7 @@ class Postgres(object):
             filter_list.append('"{}" = {}'.format(k, v))
         filter_str = 'where' + ' and '.join(filter_list) if filter_list else ''
         sql = "select {} from {} {}".format(columns_str, table, filter_str)
-        logger.info('sql: %s' %sql)
+        logger.info('sql: %s' % sql)
         cursor.execute(sql)
         res = cursor.fetchone()
         cursor.close()
@@ -427,10 +427,10 @@ class Postgres(object):
         """
         conf = self.conf
         conn = psycopg2.connect(database=conf['database'],
-                                    user=conf['user'],
-                                    password=conf['password'],
-                                    host=conf['host'],
-                                    port=int(conf['port']))
+                                user=conf['user'],
+                                password=conf['password'],
+                                host=conf['host'],
+                                port=int(conf['port']))
         cursor = conn.cursor()
         cursor.execute(sql)
         res = cursor.fetchall()
@@ -452,10 +452,10 @@ class Postgres(object):
     def over_insert_df(self, df, table_name, text_column, date_columns=None):
         conf = self.conf
         conn = psycopg2.connect(database=conf['database'],
-                                    user=conf['user'],
-                                    password=conf['password'],
-                                    host=conf['host'],
-                                    port=int(conf['port']))
+                                user=conf['user'],
+                                password=conf['password'],
+                                host=conf['host'],
+                                port=int(conf['port']))
         cursor = conn.cursor()
 
         c0 = df.columns.tolist()
@@ -464,7 +464,7 @@ class Postgres(object):
 
         if date_columns:
             text_column.extend(date_columns)
-        cursor.execute(f'DROP TABLE IF EXISTS "{table_name}";')
+        cursor.execute('DROP TABLE IF EXISTS "{}";'.format(table_name))
         count = 0
         column_list = []
 
@@ -475,28 +475,31 @@ class Postgres(object):
 
                 if not text_column:
                     text_column = column_list
-                sql_str = ','.join([f'"{j.replace(" ", "_")}" float' if j not in text_column
-                                    else f'"{j.replace(" ", "_")}" text'
+                sql_str = ','.join(['"{}" float'.format(j.replace(" ", "_")) if j not in text_column
+                                    else '"{}" text'.format(j.replace(" ", "_"))
                                     for j in column_list])
                 if not date_columns:
                     date_columns = []
                 for dd in date_columns:
-                    sql_str = sql_str.replace(f'"{dd}" text', f'"{dd}" date')
+                    sql_str = sql_str.replace(
+                        '"{}" text'.format(dd), '"{}" date'.format(dd))
                 # sql_s = re.findall(r'[^\*/:?\\|<>!]', sql_s, re.S)
                 # sql_s = "".join(sql_s)
                 # 自定义数据类型
                 if not self.is_table_exist(table_name):
-                    cursor.execute(f'create table {table_name} ({sql_str});')
-                    print('create table', sql_str)
+                    cursor.execute(
+                        'create table {} ({});'.format(table_name, sql_str))
             else:
-                sql = f"INSERT INTO {table_name} VALUES ({','.join(['%s' for i in range(len(row))])})"
+                sql = "INSERT INTO {table_name} VALUES ({})".format(
+                    ','.join(['%s' for i in range(len(row))]))
                 for i in range(len(row)):
 
                     if date_columns and column_list[i] in date_columns:
                         if isinstance(row[i], pd.Timestamp):
                             row[i] = row[i].date()
                         elif isinstance(row[i], str):
-                            row[i] = datetime.strptime(row[i], '%Y-%m-%d').date()
+                            row[i] = datetime.strptime(
+                                row[i], '%Y-%m-%d').date()
                         elif isinstance(row[i], date):
                             pass
                         elif isinstance(row[i], datetime):
@@ -517,7 +520,49 @@ class Postgres(object):
                 # print(row)
             count += 1
 
-        print(f'saved {table_name}')
         conn.commit()
         cursor.close()
-        conn.close()
\ No newline at end of file
+        conn.close()
+        logger.info('saved {}'.format(table_name))
+
+    def over_insert_df_pro(self, df, table_name, text_column, date_columns=None):
+        conf = self.conf
+        conn = psycopg2.connect(database=conf['database'],
+                                user=conf['user'],
+                                password=conf['password'],
+                                host=conf['host'],
+                                port=int(conf['port']))
+        cursor = conn.cursor()
+        cc = df.values.tolist()
+        str_list = []
+        for c in cc:
+            str_list.append(','.join([str(i) for i in c]))
+        string = '\n'.join(str_list)
+        columns = df.columns.tolist()
+        f = StringIO()  # StringIO 结构类似文件，但是内容都在内存里面
+        # 循环写入数据到内存里面， 里面每个字段用制表符\t 隔开，每一行用换行符\n 隔开
+        f.write(string)
+
+        # 最重要的一步，要把f 的游标移到第一位，write 方法后，游标会变成最尾，StringIO(**) 就不会
+        f.seek(0)
+        if not self.is_table_exist(table_name):
+            sql_list = []
+            print(columns)
+            for dd in columns:
+                if dd in date_columns:
+                    ddd = '{dd} date'.format(dd)
+                elif dd in text_column:
+                    ddd = '{} text'.format(dd)
+                else:
+                    ddd = '{} float'.format(dd)
+                sql_list.append(ddd)
+            sql_str = ','.join(sql_list)
+            cursor.execute('create table {} ({});'.format(table_name, sql_str))
+        cursor.copy_from(f, table_name,
+                         columns=columns,
+                         sep=',', null='', size=16384)  # 默认sep和null 都是none
+
+        conn.commit()
+        cursor.close()
+        conn.close()
+        logger.info('saved {}'.format(table_name))
